import React, { useState, useMemo, useEffect, Suspense } from 'react';
import { VISComponentType, VisualizationConfiguration } from '../common';
import {
  removeVisualization,
  reorderVisState,
  setActiveVisualization,
  updateVisualization,
} from '../../data-access/store/visualizationSlice';
import {
  useAppDispatch,
  useGraphQueryResult,
  useGraphQueryResultMeta,
  useML,
  useSchemaGraph,
  useSessionCache,
  useVisualization,
} from '../../data-access';
import { VisualizationManagerType } from '.';
import { SelectType, addSelect } from '../../data-access/store/interactionSlice';
import { PaohVisComponent } from '../visualizations/paohvis/paohvis';

export const Visualizations: Record<string, Function> = {
  TableVis: () => import('../visualizations/tablevis/tablevis'),
  // PaohVis: () => import('../visualizations/paohvis/paohvis'),
  PaohVis: () => Promise.resolve({ default: PaohVisComponent }),
  RawJSONVis: () => import('../visualizations/rawjsonvis/rawjsonvis'),
  NodeLinkVis: () => import('../visualizations/nodelinkvis/nodelinkvis'),
  MatrixVis: () => import('../visualizations/matrixvis/matrixvis'),
  SemanticSubstratesVis: () => import('../visualizations/semanticsubstratesvis/semanticsubstratesvis'),
  MapVis: () => import('../visualizations/mapvis/mapvis'),
};

export const VISUALIZATION_TYPES: string[] = Object.keys(Visualizations);

export const VisualizationManager = (): VisualizationManagerType => {
  const dispatch = useAppDispatch();
  const session = useSessionCache();
  const ml = useML();
  const schema = useSchemaGraph();
  const graphQueryResult = useGraphQueryResult();
  const graphMetadata = useGraphQueryResultMeta();
  const { activeVisualization, openVisualizations } = useVisualization();

  const [configuration, setConfiguration] = useState<any>();
  const [visualization, setVisualization] = useState<VISComponentType>();
  const [selected, setSelected] = useState<any>();
  const activeType = useMemo(
    () => (activeVisualization ? openVisualizations[activeVisualization]?.displayName : undefined),
    [openVisualizations, activeVisualization],
  );
  const tabs = useMemo(() => (Object.keys(openVisualizations).length ? Object.keys(openVisualizations) : []), [openVisualizations]);

  useEffect(() => {
    loadVisualization();
  }, [activeVisualization]);

  const loadVisualization = async () => {
    if (activeVisualization && Visualizations[activeVisualization]) {
      const componentModule = await Visualizations[activeVisualization]();
      const component = componentModule.default;

      if (!(activeVisualization in Object.keys(openVisualizations))) {
        // Visualization doesn't yet exist so add its configuration
        const configuration = component.configuration;
        dispatch(updateVisualization({ id: activeVisualization, settings: configuration }));
        setConfiguration(configuration);
      } else {
        setConfiguration(openVisualizations[activeVisualization]);
      }

      setVisualization(component);
    }
  };

  const changeActive = (id: string) => {
    dispatch(setActiveVisualization(id));
  };

  const deleteVisualization = (id: string) => {
    dispatch(removeVisualization(id));
    if (Object.keys(openVisualizations).length > 0) {
      const newActive = tabs.find((v: string) => id !== v);
      changeActive(newActive || '');
    }
  };

  const reorderVisualizations = ({ draggedVisId, dropVisId }: { draggedVisId: string; dropVisId: string }) => {
    const settingsCopy = { ...openVisualizations };
    const keys = Object.keys(settingsCopy);
    const draggedIndex = keys.indexOf(draggedVisId);
    const dropIndex = keys.indexOf(dropVisId);

    if (draggedIndex !== -1 && dropIndex !== -1) {
      keys.splice(dropIndex, 0, draggedVisId);
      const newSettings: { [id: string]: VisualizationConfiguration } = {};
      keys.forEach((key) => {
        newSettings[key] = settingsCopy[key];
      });

      dispatch(reorderVisState(newSettings));
    }
  };

  const handleSelect = (item: SelectType | undefined) => {
    dispatch(addSelect(item));
  };

  const updateSettings = (newSettings: Record<string, any>) => {
    if (activeVisualization) {
      const updatedSettings = { ...configuration, ...newSettings };
      setConfiguration(updatedSettings);
      dispatch(updateVisualization({ id: activeVisualization, settings: updatedSettings }));
    }
  };

  const renderSettings = () => {
    return (
      visualization?.settings &&
      configuration && (
        <visualization.settings configuration={configuration} graphMetadata={graphMetadata} updateSettings={updateSettings} />
      )
    );
  };

  // TODO: we should remove the renderable part of this useFunction into its own component, since this here is an anti-pattern
  const renderComponent = () => {
    return (
      <Suspense fallback={<div>Loading...</div>}>
        {visualization?.component && configuration && (
          <visualization.component
            data={graphQueryResult}
            schema={schema}
            ml={ml}
            configuration={configuration}
            dispatch={dispatch}
            handleSelect={handleSelect}
            graphMetadata={graphMetadata}
            updateSettings={updateSettings}
            handleHover={() => {}}
          />
        )}
      </Suspense>
    );
  };

  return {
    renderComponent,
    renderSettings,
    activeVisualization,
    activeType,
    tabs,
    changeActive,
    reorderVisualizations,
    deleteVisualization,
  };
};
