import React, { useEffect, useState, useMemo } from 'react';
import CustomLine from './CustomLine';
import { intersectionElements } from '../utils/utils';
import { PaohvisDataPaginated, RowInformation } from '../types';
import { ArrowDownward, ArrowUpward, Sort } from '@mui/icons-material';
import { select } from 'd3';

interface HyperEdgeRangesBlockProps {
  dataModel: PaohvisDataPaginated;
  rowHeight: number;
  yOffset: number;
  rowLabelColumnWidth: number;
  classTopTextColumns: string;
  currentPageRows?: {
    startIndexRow: number;
    endIndexRow: number;
  } | null;
  onMouseEnterHyperEdge: (event: React.MouseEvent<SVGGElement, MouseEvent>) => void;
  onMouseLeaveHyperEdge: () => void;
  rowsMaxPerPage: number;
  marginText: number;
  sortState: string;
  numRows: number;
  columnHeaderInformation: RowInformation;
  widthColumns: number;
  headerState: string;
  handleClickHeaderSorting: (event: React.MouseEvent<SVGGElement, MouseEvent>) => void;
}

export const HyperEdgeRangesBlock: React.FC<HyperEdgeRangesBlockProps> = ({
  dataModel,
  rowHeight,
  yOffset,
  rowLabelColumnWidth,
  classTopTextColumns,
  currentPageRows,
  rowsMaxPerPage,
  marginText,
  widthColumns,
  numRows,
  sortState,
  headerState,
  columnHeaderInformation,
  onMouseEnterHyperEdge,
  onMouseLeaveHyperEdge,
  handleClickHeaderSorting,
}) => {
  const [linePositions, setLinePositions] = useState<{ y0: number; y1: number; valid: boolean }[]>([]);

  useEffect(() => {
    if (currentPageRows && dataModel) {
      const newLinePositions = dataModel.pageData.hyperEdgeRanges.map((hyperEdgeRange) => {
        //console.log(hyperEdgeRange);
        return intersectionElements([currentPageRows.startIndexRow, currentPageRows.endIndexRow], hyperEdgeRange.hyperEdges.indices);
      });
      setLinePositions(newLinePositions);
    } else {
      setLinePositions([]);
    }
  }, [currentPageRows, dataModel]);

  const accumulatedWidthHeaders = useMemo(() => {
    const accumulatedWidths: number[] = [0];
    let sum = 0;

    columnHeaderInformation.forEach((column, index) => {
      sum += column.width;
      if (index !== columnHeaderInformation.length - 1) {
        accumulatedWidths.push(sum);
      }
    });

    return accumulatedWidths;
  }, [columnHeaderInformation]);

  const [isHovered, setIsHovered] = useState(false);
  const [hoverRowIndex, setHoverRowIndex] = useState<number | null>(null);
  const [iconComponents, setIconComponents] = useState<{ [key: number]: JSX.Element }[]>(Array(columnHeaderInformation.length).fill({}));
  const [iconColors, setIconColors] = useState<string[]>([]);

  useEffect(() => {
    const iconColorsTemporal: string[] = [];

    const updatedIconComponents = columnHeaderInformation.map((row, indexRows) => {
      let iconComponent: JSX.Element = <></>;
      if (row.header == headerState) {
        switch (sortState) {
          case 'asc':
            iconComponent = <ArrowUpward />;
            break;
          case 'desc':
            iconComponent = <ArrowDownward />;
            break;
          case 'original':
            iconComponent = <Sort />;

            break;
          default:
            iconComponent = <></>;
        }
        iconColorsTemporal.push('hsl(var(--clr-sec--800))');
      } else {
        iconComponent = <></>;
        iconColorsTemporal.push('hsl(var(--clr-sec--500))');
      }

      if (indexRows == hoverRowIndex && headerState != row.header) {
        iconComponent = <Sort />;
      }
      return { [indexRows]: iconComponent };
    });

    setIconColors(iconColorsTemporal);
    setIconComponents(updatedIconComponents);
  }, [sortState, headerState, hoverRowIndex]);

  return (
    <>
      <g key={'hyperEdgeInformationTop'} className="hyperEdgeInformation">
        <g key={'hyperEdgeLinesRows'} transform={`translate(${rowLabelColumnWidth},${yOffset})`}>
          {[...Array(numRows)].map((_, i) => (
            <React.Fragment key={`horizontalLineRows-${i}`}>
              {i === 0 && (
                <CustomLine
                  key={`horizontalLineRowsTop-${i}`}
                  x1={0}
                  x2={dataModel.pageData.hyperEdgeRanges.length * rowHeight}
                  y1={-yOffset}
                  y2={-yOffset}
                  strokeWidth={5}
                  fill="hsl(var(--clr-pri--600))"
                />
              )}
              {i === numRows - 1 && (
                <CustomLine
                  key={`horizontalLineRowsExtra-${i}`}
                  x1={0}
                  x2={dataModel.pageData.hyperEdgeRanges.length * rowHeight}
                  y1={rowHeight * (i + 1)}
                  y2={rowHeight * (i + 1)}
                  strokeWidth={1}
                  fill="hsl(var(--clr-pri--600))"
                />
              )}

              <CustomLine
                key={`horizontalLineRowsBottom-${i}`}
                x1={0}
                x2={dataModel.pageData.hyperEdgeRanges.length * rowHeight}
                y1={rowHeight * i}
                y2={rowHeight * i}
                strokeWidth={1}
                fill="hsl(var(--clr-pri--600))"
              />
            </React.Fragment>
          ))}
        </g>
        <g
          key={'hyperEdgeInformation'}
          className="hyperEdgeInformation text-columns "
          transform={'translate(' + rowLabelColumnWidth + ',' + widthColumns + ')'}
        >
          {columnHeaderInformation[0] &&
            columnHeaderInformation[0].data.map((rowLabel, indexRows) => (
              <g
                key={'colsLabel col-' + indexRows}
                className={'colsLabel col-' + indexRows}
                transform={'translate(' + indexRows * rowHeight + ',' + 0 + ')rotate(-90,0,0)'}
              >
                {columnHeaderInformation.map((row, index) => (
                  <g key={'text-col-' + index} transform={'translate(' + accumulatedWidthHeaders[index] + ',' + 0 + ')'}>
                    <rect width={row.width} height={rowHeight} fill={'hsl(var(--clr-sec--0))'} strokeWidth={0}></rect>
                    <text x={row.width * marginText} y={rowHeight / 2} dy="0" dominantBaseline="middle" className={classTopTextColumns}>
                      {row.data[indexRows]}
                    </text>
                  </g>
                ))}
              </g>
            ))}
        </g>

        <g
          key={'columnInformationHeaders'}
          className={'columnInformationHeaders'}
          transform={'translate(' + (rowLabelColumnWidth - rowHeight) + ',' + widthColumns + ')'}
        >
          {columnHeaderInformation.map((headerData, headerIndex) => (
            <g
              key={'gHeadersColumns-' + headerIndex}
              className={`headersCols-${headerData.header} cursor-pointer`}
              transform={'translate(' + 0 + ',' + -accumulatedWidthHeaders[headerIndex] + ')rotate(-90, 0,0)'}
              onClick={(event) => {
                handleClickHeaderSorting(event);
              }}
              onMouseEnter={(event) => {
                setHoverRowIndex(headerIndex);
                setIsHovered(true);
                select(event.currentTarget).select('rect').attr('fill', 'hsl(var(--clr-sec--300))');
              }}
              onMouseLeave={(event) => {
                setIsHovered(sortState !== 'original');
                setHoverRowIndex(null);
                select(event.currentTarget).select('rect').attr('fill', 'hsl(var(--clr-sec--200))');
              }}
            >
              <rect width={headerData.width} height={rowHeight} fill={'hsl(var(--clr-sec--200))'} opacity={1.0} strokeWidth={0}></rect>
              <text x={marginText * headerData.width} y={0.5 * rowHeight} dy="0" dominantBaseline="middle" className={classTopTextColumns}>
                {headerData.header}
              </text>
              {iconComponents[headerIndex] && isHovered && (
                <svg
                  xmlns="http://www.w3.org/2000/svg"
                  width={2 * headerData.width - rowHeight}
                  height={24}
                  style={{ color: iconColors[headerIndex], stroke: 'none' }}
                >
                  {iconComponents[headerIndex][headerIndex]}
                </svg>
              )}
              <CustomLine
                x1={headerData.width}
                x2={headerData.width}
                y1={0}
                y2={rowHeight * (dataModel.pageData.hyperEdgeRanges.length + 1)}
                strokeWidth={1}
                fill="hsl(var(--clr-pri--600))"
              />
            </g>
          ))}
        </g>
        {dataModel.pageData.hyperEdgeRanges.map((hyperEdgeRange, indexHyperEdgeRange) => (
          <React.Fragment key={`fragment-${indexHyperEdgeRange}`}>
            <g
              key={'hyperEdgeBlockLinesRef-' + indexHyperEdgeRange}
              className={'hyperEdgeBlockLinesRef hyperEdgeLines-col-' + indexHyperEdgeRange}
            >
              <g
                key={'hyperEdgeBlockLinesRefTransformed-' + indexHyperEdgeRange}
                transform={'translate(' + rowLabelColumnWidth + ',' + widthColumns + ')'}
              >
                {currentPageRows &&
                  dataModel.pageData.hyperEdgeRanges.map((row, index) => (
                    <g
                      key={'´gColsLinesTable-' + indexHyperEdgeRange + '-' + index}
                      transform={'translate(' + index * rowHeight + ',' + 0 + ')rotate(-90,0,0)'}
                    >
                      {index == 0 ? (
                        <CustomLine x1={-rowHeight * numRows} x2={0} y1={0} y2={0} strokeWidth={0.5} fill="hsl(var(--clr-pri--600))" />
                      ) : (
                        <CustomLine
                          x1={-rowHeight * numRows}
                          x2={yOffset}
                          y1={0}
                          y2={0}
                          strokeWidth={0.5}
                          fill="hsl(var(--clr-pri--600))"
                        />
                      )}
                    </g>
                  ))}
                <g transform={'translate(' + (dataModel.pageData.hyperEdgeRanges.length - 1) * rowHeight + ',' + 0 + ')rotate(-90,0,0)'}>
                  <CustomLine
                    x1={-rowHeight * numRows}
                    x2={yOffset}
                    y1={rowHeight}
                    y2={rowHeight}
                    strokeWidth={0.5}
                    fill="hsl(var(--clr-pri--600))"
                  />
                </g>
              </g>
            </g>

            <g
              className={'hyperEdgeBlock hyperEdge-col-' + indexHyperEdgeRange}
              key={'hyperEdgeBlockddd hyperEdge-col-' + indexHyperEdgeRange}
              onMouseEnter={onMouseEnterHyperEdge}
              onMouseLeave={onMouseLeaveHyperEdge}
            >
              <g
                key={'groupBlockExtra-' + indexHyperEdgeRange}
                transform={`translate(${rowLabelColumnWidth + indexHyperEdgeRange * rowHeight + 0.5 * rowHeight},${yOffset + 0.5 * rowHeight})`}
              >
                {currentPageRows && linePositions[indexHyperEdgeRange]?.valid && (
                  <line
                    key={'hyperRangeBlockLine line_' + indexHyperEdgeRange}
                    x1={0}
                    y1={(linePositions[indexHyperEdgeRange].y0 - currentPageRows.startIndexRow) * rowHeight}
                    x2={0}
                    y2={(linePositions[indexHyperEdgeRange].y1 - currentPageRows.startIndexRow) * rowHeight}
                    strokeWidth={1.5}
                    stroke="hsl(var(--clr-sec--800))"
                  />
                )}
              </g>

              <g
                className={'hyperEdgeBlockCircles'}
                key={'hyperEdgeBlockCircles-' + indexHyperEdgeRange}
                transform={`translate(${rowLabelColumnWidth + indexHyperEdgeRange * rowHeight + 0.5 * rowHeight},${yOffset + 0.5 * rowHeight})`}
              >
                {currentPageRows &&
                  hyperEdgeRange.hyperEdges.indices
                    .filter((valueIndex) => valueIndex >= currentPageRows.startIndexRow && valueIndex < currentPageRows.endIndexRow)
                    .map((valueIndex: number, indexColumn: number) => (
                      <circle
                        key={'circleBlock-' + valueIndex + '_' + indexColumn}
                        className={`circle-${valueIndex}`}
                        cx={0}
                        cy={(valueIndex - currentPageRows.startIndexRow) * rowHeight}
                        r={rowHeight * 0.25}
                        fill="white"
                        strokeWidth={rowHeight * 0.03}
                        stroke="black"
                      />
                    ))}
              </g>
            </g>
          </React.Fragment>
        ))}
      </g>
    </>
  );
};
