import React, { useEffect, useState } from 'react';
import CustomLine from './CustomLine';
import { RowInformation } from '../types';
import { ArrowDownward, ArrowUpward, Sort } from '@mui/icons-material';
import { select, selectAll } from 'd3';

interface RowLabelsProps {
  dataRows: RowInformation;
  rowHeight: number;
  yOffset: number;
  rowLabelColumnWidth: number;
  classTopTextColumns: string;
  marginText: number;
  sortState: string;
  headerState: string;
  onMouseEnterRowLabels: (event: React.MouseEvent<SVGGElement, MouseEvent>) => void;
  onMouseLeaveRowLabels: () => void;
  handleClickHeaderSorting: (event: React.MouseEvent<SVGGElement, MouseEvent>) => void;
}

export const RowLabels: React.FC<RowLabelsProps> = ({
  dataRows,
  rowHeight,
  yOffset,
  rowLabelColumnWidth,
  classTopTextColumns: classTopTextColums,
  marginText,
  sortState,
  headerState,
  onMouseEnterRowLabels,
  onMouseLeaveRowLabels,
  handleClickHeaderSorting,
}) => {
  const accumulatedWidthHeaders = [0];
  let sum = 0;
  dataRows.forEach((row, index) => {
    sum += row.width;
    if (index !== dataRows.length - 1) {
      accumulatedWidthHeaders.push(sum);
    }
  });

  const [isHovered, setIsHovered] = useState(false);
  const [hoverRowIndex, setHoverRowIndex] = useState<number | null>(null);
  const [iconComponents, setIconComponents] = useState<{ [key: number]: JSX.Element }[]>(Array(dataRows.length).fill({}));
  const [iconColors, setIconColors] = useState<string[]>([]);

  useEffect(() => {
    const iconColorsTemporal: string[] = [];

    const updatedIconComponents = dataRows.map((row, indexRows) => {
      let iconComponent: JSX.Element = <></>;
      if (row.header == headerState) {
        switch (sortState) {
          case 'asc':
            iconComponent = <ArrowUpward />;
            break;
          case 'desc':
            iconComponent = <ArrowDownward />;
            break;
          case 'original':
            iconComponent = <Sort />;
            break;
          default:
            iconComponent = <></>;
        }
        iconColorsTemporal.push('hsl(var(--clr-sec--800))');
      } else {
        iconComponent = <></>;
        iconColorsTemporal.push('hsl(var(--clr-sec--500))');
      }

      if (indexRows == hoverRowIndex && headerState != row.header) {
        iconComponent = <Sort />;
      }

      return { [indexRows]: iconComponent };
    });

    setIconColors(iconColorsTemporal);
    setIconComponents(updatedIconComponents);
  }, [sortState, headerState, hoverRowIndex]);
  return (
    <>
      <g key={'rowLabelsInformation'} className="rowLabelsInformation">
        {dataRows[0] &&
          dataRows[0].data.map((rowLabel, indexRows) => (
            <g key={indexRows} transform={'translate(0,' + (yOffset + indexRows * rowHeight) + ')'}>
              <g
                key={indexRows}
                className={'rowsLabel row-' + indexRows}
                onMouseEnter={onMouseEnterRowLabels}
                onMouseLeave={onMouseLeaveRowLabels}
              >
                {dataRows.map((row, index) => (
                  <g key={'´gRowTable-' + index} transform={'translate(' + accumulatedWidthHeaders[index] + ',' + 0 + ')'}>
                    <rect
                      width={row.width}
                      height={rowHeight}
                      fill={indexRows % 2 === 0 ? 'hsl(var(--clr-sec--50))' : 'hsl(var(--clr-sec--0))'}
                      strokeWidth={0}
                    ></rect>
                    <text x={row.width * marginText} y={rowHeight / 2} dy="0" dominantBaseline="middle" className={classTopTextColums}>
                      {row.data[indexRows]}
                    </text>
                  </g>
                ))}
              </g>
              <CustomLine x1={0} x2={rowLabelColumnWidth} y1={0} y2={0} strokeWidth={1} fill="hsl(var(--clr-pri--600))" />
              {indexRows !== dataRows[0].data.length && (
                <CustomLine x1={0} x2={rowLabelColumnWidth} y1={rowHeight} y2={rowHeight} strokeWidth={1} fill="hsl(var(--clr-pri--600))" />
              )}
            </g>
          ))}

        <g key={'rowInformationHeadersLines'} className={'rowInformationHeadersLines'}>
          <CustomLine
            x1={0}
            x2={rowLabelColumnWidth - rowHeight}
            y1={yOffset - rowHeight}
            y2={yOffset - rowHeight}
            strokeWidth={1}
            fill="hsl(var(--clr-pri--600))"
          />
          <CustomLine
            x1={rowLabelColumnWidth - rowHeight}
            x2={rowLabelColumnWidth}
            y1={yOffset - rowHeight}
            y2={yOffset + 0 * rowHeight}
            strokeWidth={1}
            fill="hsl(var(--clr-pri--600))"
          />
        </g>
        <g key={'rowInformationHeaders'} className={'rowInformationHeaders'}>
          {dataRows.map((row, indexRows) => (
            <g key={'headersContent-' + indexRows}>
              <g
                key={'gHeadersRows-' + indexRows}
                className={`headersRows-${row.header} cursor-pointer`}
                transform={'translate(' + accumulatedWidthHeaders[indexRows] + ',' + (yOffset - rowHeight) + ')'}
                onClick={(event) => {
                  handleClickHeaderSorting(event);
                }}
                onMouseEnter={(event) => {
                  setHoverRowIndex(indexRows);
                  setIsHovered(true);
                  select(event.currentTarget).select('rect').attr('fill', 'hsl(var(--clr-sec--300))');
                }}
                onMouseLeave={(event) => {
                  setIsHovered(sortState !== 'original');
                  setHoverRowIndex(null);
                  select(event.currentTarget).select('rect').attr('fill', 'hsl(var(--clr-sec--200))');
                }}
              >
                <rect width={row.width} height={rowHeight} fill={'hsl(var(--clr-sec--200))'} opacity={1.0} strokeWidth={0}></rect>
                <text x={marginText * row.width} y={0.5 * rowHeight} dy="0" dominantBaseline="middle" className={classTopTextColums}>
                  {row.header}
                </text>
                {iconComponents[indexRows] && isHovered && (
                  <svg
                    xmlns="http://www.w3.org/2000/svg"
                    width={2 * row.width - rowHeight}
                    height={24}
                    style={{ color: iconColors[indexRows], stroke: 'none' }}
                  >
                    {iconComponents[indexRows][indexRows]}
                  </svg>
                )}
              </g>
              <CustomLine
                key={'row-verticalLines-' + indexRows}
                x1={accumulatedWidthHeaders[indexRows]}
                x2={accumulatedWidthHeaders[indexRows]}
                y1={yOffset - rowHeight}
                y2={yOffset + dataRows[0].data.length * rowHeight}
                strokeWidth={1}
                fill="hsl(var(--clr-pri--600))"
              />
            </g>
          ))}
        </g>
      </g>
    </>
  );
};
